"""distutils.pypirc

Provides the PyPIRcCommand class, the base class for the command classes
that uses .pypirc in the distutils.command package.
"""
import os
from ConfigParser import ConfigParser

from distutils.core import Command 

class PyPIRCCommand(Command):

    DEFAULT_REPOSITORY = 'http://www.python.org/pypi'
    DEFAULT_REALM = 'pypi'
    repository = ''
    realm = ''

    def _store_pypirc(self, username, password):
        """stores the choice"""
        rc = os.path.join(os.environ['HOME'], '.pypirc')
        f = open(rc, 'w')
        try:
            f.write(('[pypirc]\n    servers = default\n '
                     '[default]\nusername:%s\npassword:%s\n') % \
                        (username, password))
        finally:
            f.close()
        try:
            os.chmod(rc, 0600)
        except:     # XXX ????
            pass

    def _read_pypirc(self):
        """Reads the .pypirc file."""
        if os.environ.has_key('HOME'): 
            rc = os.path.join(os.environ['HOME'], '.pypirc')
            if os.path.exists(rc):
                print 'Using PyPI login from %s' % rc
                repository = self.repository or self.DEFAULT_REPOSITORY
                realm = self.realm or self.DEFAULT_REALM

                config = ConfigParser()
                config.read(rc)
                sections = config.sections()
                
                if 'distutils' in sections:
                    # let's get the list of servers
                    index_servers = config.get('distutils', 'index-servers')
                    _servers = [server.strip() for server in 
                                index_servers.split('\n') 
                                if server.strip() != '']
                    if _servers == []:
                        # nothing set, let's try to get the defaut pypi
                        if 'pypi' in sections:
                            _servers = ['pypi']
                        else:
                            # the file is not properly defined, returning
                            # an empty dict
                            return {}
                    for server in _servers:
                        current = {'server': server}
                        current['username'] = config.get(server, 'username')
                        current['password'] = config.get(server, 'password')
                        
                        # optional params
                        for key, default in (('repository', 
                                              self.DEFAULT_REPOSITORY),
                                             ('realm', self.DEFAULT_REALM)):
                            if config.has_option(server, key):
                                current[key] = config.get(server, key)
                            else:
                                current[key] = default
                        if (current['server'] == repository or 
                            current['repository'] == repository):
                            return current
                elif 'server-login' in sections:
                    # old format
                    server = 'server-login'
                    if config.has_option(server, 'repository'):
                        repository = config.get(server, 'repository')
                    else:
                        repository = self.DEFAULT_REPOSITORY
                    return {'username': config.get(server, 'username'),
                            'password': config.get(server, 'password'),
                            'repository': repository,
                            'server': server,
                            'realm': self.DEFAULT_REALM}
                    
        return {}





"""Tests for distutils.pypirc.pypirc."""
import sys
import os
import unittest

from iw.dist.core import PyPIRCCommand
from distutils.core import Distribution

from distutils.tests import support

PYPIRC = """\
[distutils]

index-servers = 
    server1
    server2

[server1]
username:me
password:secret

[server2]
username:meagain
password: secret
repository:http://another.pypi/
"""

PYPIRC_OLD = """\
[server-login]
username:tarek
password:secret
"""

class PyPIRCCommandTestCase(support.TempdirManager, unittest.TestCase):

    def setUp(self):
        """Patches the environment."""
        if os.environ.has_key('HOME'):
            self._old_home = os.environ['HOME']
        else:
            self._old_home = None
        curdir = os.path.dirname(__file__)
        os.environ['HOME'] = curdir
        self.rc = os.path.join(curdir, '.pypirc')
    
    def tearDown(self):
        """Removes the patch."""
        if self._old_home is None:
            del os.environ['HOME']
        else:
            os.environ['HOME'] = self._old_home
        if os.path.exists(self.rc):
            os.remove(self.rc)

    def test_server_registration(self):
        # This test makes sure PyPIRCCommand knows how to:
        # 1. handle several sections in .pypirc
        # 2. handle the old format
    
        # new format
        f = open(self.rc, 'w')
        f.write(PYPIRC)
        f.close()

        dist = Distribution()
        class command(PyPIRCCommand):
            def __init__(self, dist):
                PyPIRCCommand.__init__(self, dist)
            def initialize_options(self):
                pass
            finalize_options = initialize_options

        cmd = command(dist)
        config = cmd._read_pypirc()

        config = config.items()
        config.sort()
        waited = [('password', 'secret'), ('realm', 'pypi'), 
                  ('repository', 'http://www.python.org/pypi'), 
                  ('server', 'server1'), ('username', 'me')]
        self.assertEquals(config, waited)

        # old format
        f = open(self.rc, 'w')
        f.write(PYPIRC_OLD)
        f.close()
        
        config = cmd._read_pypirc() 
        config = config.items()
        config.sort()
        waited = [('password', 'secret'), ('realm', 'pypi'), 
                  ('repository', 'http://www.python.org/pypi'), 
                  ('server', 'server-login'), ('username', 'tarek')]
        self.assertEquals(config, waited)

    
def test_suite():
    return unittest.makeSuite(PyPIRCCommandTestCase)

if __name__ == "__main__":
    unittest.main(defaultTest="test_suite")
